/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//    Receive frames from Fd and stored them in a DMT shared memory.
//
#include "lsmp_prod.hh"
#include "SigFlag.hh"
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <string>

extern "C" {
#include "CmMessage.h"
    char* FdGetFrameFromCm(double timeOut, int *nBytes, int *gps);
}

using namespace std;

//======================================  Main function.
int 
main(int argc, const char *argv[]) {
    string name = "LHOV1In";
    int debug=0;
    string part;
    const char* cPart =  getenv("LIGOSMPART");
    if (cPart) part = cPart;

    for (int i=1; i<argc; ++i) {
        string argi = argv[i];
        if (argi == "-d") {
            debug = 1;
        } else if (argi == "-n" || argi == "--name") {
            name = argv[++i];
        } else if (argi == "-p" || argi == "--partition") {
            part = argv[++i];
        } else {
            cerr << "Invalid argument: " << argi << endl;
            exit(2);
	}
    }
 
    //----------------------------------  See if we can open the cm connection
    cout << "Cm name: " << name << endl;
    if( !CmMessageOpenServer ((char *) name.c_str())) {
	cerr << " Cannot open Cm" << endl;
	exit(1);
    } else {
	cout << " Cm OK" << endl;
    }

    CmConnectSetSocketSize(262144, 262144, "*");
    
    //----------------------------------  Open the partition 
    LSMP_PROD smprod(part.c_str());
    if (!smprod) {
	cerr << "FdtoDMT unable to attach partition " << part << endl;
	exit(2);
    }

    //----------------------------------  Handle signals
    SigFlag mTerm;
    mTerm.add(SIGTERM);
    mTerm.add(SIGHUP);
    mTerm.add(SIGINT);

    //----------------------------------  Loop on frames
    int ret = 0;
    while (!mTerm) {

        int nBytes = 0;
	int gps    = 0;
        char* buff = FdGetFrameFromCm(1., &nBytes, &gps);
	if(buff == NULL) continue;

        //------  "buff" contains the image of a frame written on a file.
        //----------- the size of "buff" is "nBytes"
	//--- we could put "buff" here in a DMT shared memory free(buff);
        if (debug) cout << "received " << nBytes <<" Bytes" << endl;

	//----------------------------------  Get a free buffer from the 
	//                                    partition. Check that it is OK.
	char* smbuff = smprod.get_buffer();
	if (!smbuff) {
	    cerr << "get_buffer failed in partition " << part << endl;
	    ret = 3;
	    break;
	} else if (smprod.getBufferLength() < nBytes) {
	    cerr << "Frame length (" << nBytes 
		 << ") is greater than the output buffer size ("
		 << smprod.getBufferLength() << ")." << endl;
	    nBytes = smprod.getBufferLength();
	}

	//----------------------------------  Copy the data to shared memory.
	memcpy(smbuff, buff, nBytes);

	//----------------------------------  Get the frame ID (gps)
	smprod.SetID(gps);

	//----------------------------------  Release the buffer
	smprod.release(nBytes);

	//----------------------------------  Free the local buffer
	free(buff);

    }
    return ret;
}
